/**********************************************************************
 *
 * Copyright(c) 2008 Imagination Technologies Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful but, except
 * as otherwise stated in writing, without any warranty; without even the
 * implied warranty of merchantability or fitness for a particular purpose.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * The full GNU General Public License is included in this distribution in
 * the file called "COPYING".
 *
 * Contact Information:
 * Imagination Technologies Ltd. <gpl-support@imgtec.com>
 * Home Park Estate, Kings Langley, Herts, WD4 8LZ, UK
 *
 ******************************************************************************/

#ifndef __LINUXFB_H__
#define __LINUXFB_H__

/* max number of overlays to which a framebuffer data can be direct */
#define OMAPFB_MAX_OVL_PER_FB 3

extern IMG_BOOL PVRGetDisplayClassJTable(PVRSRV_DC_DISP2SRV_KMJTABLE *psJTable);

typedef void * LINUXFB_HANDLE;

typedef enum tag_omap_bool
{
	LINUXFB_FALSE = 0,
	LINUXFB_TRUE  = 1,
} LINUXFB_BOOL, *LINUXFB_PBOOL;

typedef struct LINUXFB_BUFFER_TAG
{
	unsigned long                ulBufferSize;
	IMG_SYS_PHYADDR              sSysAddr;
	IMG_CPU_VIRTADDR             sCPUVAddr;
	PVRSRV_SYNC_DATA*            psSyncData;
	struct LINUXFB_BUFFER_TAG*   psNext;

} LINUXFB_BUFFER;

typedef struct LINUXFB_FLIP_ITEM_TAG
{
	LINUXFB_HANDLE      hCmdComplete;
	unsigned long    ulSwapInterval;
	LINUXFB_BOOL        bValid;
	LINUXFB_BOOL        bFlipped;
	LINUXFB_BOOL        bCmdCompleted;
	IMG_SYS_PHYADDR* sSysAddr;

} LINUXFB_FLIP_ITEM;

typedef struct PVRPDP_SWAPCHAIN_TAG
{
	unsigned int                    uiSwapChainID;
	unsigned long                   ulBufferCount;
	LINUXFB_BUFFER*                 psBuffer;
	LINUXFB_FLIP_ITEM*              psFlipItems;
	unsigned long                   ulInsertIndex;
	unsigned long                   ulRemoveIndex;
	PVRSRV_DC_DISP2SRV_KMJTABLE*	psPVRJTable;
	LINUXFB_BOOL                       bFlushCommands;
	unsigned long                   ulSetFlushStateRefCount;
	LINUXFB_BOOL                       bBlanked;
	spinlock_t*                     psSwapChainLock;
	void*                           pvDevInfo;

} LINUXFB_SWAPCHAIN;

typedef struct LINUXFB_FBINFO_TAG
{
	unsigned long       ulFBSize;
	unsigned long       ulBufferSize;
	unsigned long       ulRoundedBufferSize;
	unsigned long       ulWidth;
	unsigned long       ulHeight;
	unsigned long       ulByteStride;
	IMG_SYS_PHYADDR     sSysAddr;
	IMG_CPU_VIRTADDR    sCPUVAddr;
	PVRSRV_PIXEL_FORMAT ePixelFormat;
	int                 iFBId;
}LINUXFB_FBINFO;

typedef struct LINUXFB_DEVINFO_TAG
{
	unsigned int                    uiSwapChainID;
	IMG_UINT32                      uDeviceID;
	LINUXFB_BUFFER                  sSystemBuffer;
	PVRSRV_DC_DISP2SRV_KMJTABLE	sPVRJTable;
	PVRSRV_DC_SRV2DISP_KMJTABLE	sDCJTable;
	LINUXFB_FBINFO                  sFBInfo;
	LINUXFB_SWAPCHAIN*              psSwapChain;
	LINUXFB_BOOL                       bFlushCommands;
	struct fb_info*                 psLINFBInfo;
	struct notifier_block           sLINNotifBlock;
	LINUXFB_BOOL                       bDeviceSuspended;
	struct mutex                    sSwapChainLockMutex;
	IMG_DEV_VIRTADDR	        sDisplayDevVAddr;
	DISPLAY_INFO                    sDisplayInfo;
	DISPLAY_FORMAT                  sDisplayFormat;
	DISPLAY_DIMS                    sDisplayDim;
	struct workqueue_struct*        sync_display_wq;
	struct work_struct	        sync_display_work;
#if defined(SUPPORT_DRI_DRM)
	LINUXFB_BOOL			bLeaveVT;
#endif
}  LINUXFB_DEVINFO;

typedef enum _LINUXFB_ERROR_
{
	LINUXFB_OK                             =  0,
	LINUXFB_ERROR_GENERIC                  =  1,
	LINUXFB_ERROR_OUT_OF_MEMORY            =  2,
	LINUXFB_ERROR_TOO_FEW_BUFFERS          =  3,
	LINUXFB_ERROR_INVALID_PARAMS           =  4,
	LINUXFB_ERROR_INIT_FAILURE             =  5,
	LINUXFB_ERROR_CANT_REGISTER_CALLBACK   =  6,
	LINUXFB_ERROR_INVALID_DEVICE           =  7,
	LINUXFB_ERROR_DEVICE_REGISTER_FAILED   =  8

} LINUXFB_ERROR;


struct linuxfb_info {
	int id;
//	u8 rotation[OMAPFB_MAX_OVL_PER_FB];
//	bool mirror;
};

struct linuxfb2_device {
	struct mutex  mtx;



};

#define	LINUXFB_PAGE_SIZE 4096
#define	LINUXFB_PAGE_MASK (LINUXFB_PAGE_SIZE - 1)
#define	LINUXFB_PAGE_TRUNC (~LINUXFB_PAGE_MASK)

#define	LINUXFB_PAGE_ROUNDUP(x) (((x)+LINUXFB_PAGE_MASK) & LINUXFB_PAGE_TRUNC)

#define DISPLAY_DEVICE_NAME "PowerVR OMAP Linux Display Driver"
#define	DRVNAME	"linuxfb"
#define	DEVNAME	DRVNAME
#define	DRIVER_PREFIX DRVNAME

#define FRAMEBUFFER_COUNT		num_registered_fb

//#define DEBUG
#ifdef	DEBUG
#define	DEBUG_PRINTK(format, ...) printk("DEBUG " DRIVER_PREFIX \
	" (%s %i): " format "\n", __func__, __LINE__, ## __VA_ARGS__)
#else
#define	DEBUG_PRINTK(format,...)
#endif

#define	WARNING_PRINTK(format, ...) printk("WARNING " DRIVER_PREFIX \
	" (%s %i): " format "\n", __func__, __LINE__, ## __VA_ARGS__)
#define	ERROR_PRINTK(format, ...) printk("ERROR " DRIVER_PREFIX \
	" (%s %i): " format "\n", __func__, __LINE__, ## __VA_ARGS__)

#define FB2OFB(fb_info) ((struct linuxfb_info *)(fb_info->par))

static inline void linuxfb_lock(struct linuxfb2_device *fbdev)
{
	mutex_lock(&fbdev->mtx);                  
}

static inline void linuxfb_unlock(struct linuxfb2_device *fbdev)
{
	mutex_unlock(&fbdev->mtx);               
}


LINUXFB_ERROR LINUXFBInit(void);
LINUXFB_ERROR LINUXFBDeinit(void);
LINUXFB_ERROR UnBlankDisplay(LINUXFB_DEVINFO *psDevInfo);
void *LINUXFBAllocKernelMem(unsigned long ulSize);
void LINUXFBFreeKernelMem(void *pvMem);
void LINUXFBPresentSync(LINUXFB_DEVINFO *psDevInfo,
	LINUXFB_FLIP_ITEM *psFlipItem);
void LINUXFBPresentSyncAddr(LINUXFB_DEVINFO *psDevInfo, unsigned long aPhyAddr);
LINUXFB_ERROR LINUXFBGetLibFuncAddr(char *szFunctionName,
	PFN_DC_GET_PVRJTABLE *ppfnFuncTable);
void LINUXFBFlip(LINUXFB_SWAPCHAIN *psSwapChain, unsigned long aPhyAddr);
#ifdef LDM_PLATFORM
void LINUXFBDriverSuspend(void);
void LINUXFBDriverResume(void);
#endif

#endif
